@@ -1,4 +1,6 @@
-# Copyright (C) 1998, DGA - part of the Transcriber program
+# RCS: @(#) $Id: Events.tcl,v 1.3 2003/08/27 10:09:47 barras Exp $
+
+# Copyright (C) 1998-2000, DGA - part of the Transcriber program
 # distributed under the GNU General Public License (see COPYING file)
 
 ################################################################
@@ -176,6 +178,7 @@ proc CreateBackground {} {
    # Update text on segmentation
    SetSegmtField seg0 [SearchSegmtId seg0 $bp] -text [TextFromSync $bp]
    DoModif "BACKGROUND"
+   UpdateSegmtView modified
 }
 
 proc InsertImage {tag img} {
@@ -409,12 +412,14 @@ proc EditEvent {tag {mode "Edit"} {sel ""}} {
 
    set w [CreateModal .evt "$mode event"]
 
-   set rads [RadioFrame $w.typ "Type" v(type,chosen) {
-      "Noise" "Comment" "\t" "Pronounce" "Lexical" "Language"
-   } {
-      "noise" "comment" "\t" "pronounce" "lexical" "language"
-   }]
-   
+   if {$v(chatMode)} {
+     set lstnam {"Noise" "Comment" "Dependent" "Header" "Scope" "\t" "Pronounce" "Lexical" "Language"}
+     set lstval {"noise" "comment"  "dependent" "header" "scope" "\t" "pronounce" "lexical" "language"}
+   } else {
+     set lstnam {"Noise" "Comment" "\t" "Pronounce" "Lexical" "Language"}
+     set lstval {"noise" "comment" "\t" "pronounce" "lexical" "language"}
+   }
+   set rads [RadioFrame $w.typ "Type" v(type,chosen) $lstnam $lstval]
 
    set v(desc,chosen) [$tag getAttr "desc"]
 
@@ -488,6 +493,15 @@ proc TraceEvent {w e args} {
       "language" {
 	 SetMenuEvent $e language
       }
+      "dependent" {
+	 SetMenuEvent $e dependent
+      }
+      "scope" {
+	 SetMenuEvent $e scope
+      }
+      "header" {
+	 SetMenuEvent $e header
+      }
       "pronounce" {
 	 SetMenuEvent $e pronounce
       }
@@ -611,11 +625,14 @@ proc ConfigureEvents {} {
 
    set h [frame $g.fr2 -relief raised -bd 1]
    pack $h -fill both -expand true -side left
-   foreach title {
-    "Comment" "Noise" "Pronounce" "Lexical" "Language"
-   } var {
-      "comment" "noise" "pronounce" "lexical" "language"
-   } {
+   if {$v(chatMode)} {
+     set lstname {"Comment" "Noise" "Pronounce" "Lexical" "Language" "Dependent" "Header" "Scope"}
+     set lstval {"comment" "noise" "pronounce" "lexical" "language" "dependent" "header" "scope"}
+   } else {
+     set lstnam {"Comment" "Noise" "Pronounce" "Lexical" "Language"}
+     set lstval {"comment" "noise" "pronounce" "lexical" "language"}
+   }
+   foreach title $lstnam var $lstval {
       set e [EntryFrame $h.$var $title v(event,$var)]
       $e conf -width 10
       pack $e -expand 0 -side right
@@ -646,9 +663,83 @@ proc ConfEventName {type title} {
 	    set v($type) [ListEditor $v($type) $title {"Code" "Language"}]
 	    UpdateLangList
 	 }
+	 "dependent" {
+	    set v($type) [ListEditor $v($type) $title {"Code" "Dependent"}]
+	    UpdateDepList
+	 }
+	 "header" {
+	    set v($type) [ListEditor $v($type) $title {"Code" "Header"}]
+	    UpdateHeaderList
+	 }
+	 "scope" {
+	    set v($type) [ListEditor $v($type) $title {"Code" "Scope"}]
+	    UpdateScopeList
+	 }
 	 default {
 	    set v($type) [ListEditor $v($type) $title {"Value" "Description"}]
 	 }
       }
    }
 }
+
+
+################################################################
+# added by Zhibiao
+################################################################
+
+# Insert Dependent in text editor
+proc InsertDependent {} {
+   global v
+   set t $v(tk,edit)-bis
+
+   set a [$t index insert]
+    regsub {.[0-9]+$} $a "" line 
+    
+    tkTextSetCursor $t "$line.end"
+    set a [$t index insert]
+    regsub {^[0-9]+.} $a "" lineend 
+    set lastone [expr  $lineend- 1]
+    set lastchar [$t get $line.$lastone $line.end]
+    tkTextSetCursor $t $line.$lastone
+    tkTextInsert $t "$lastchar "
+    set lastone [expr $lastone + 2]
+    tkTextSetCursor $t $line.$lastone
+    $t delete insert
+    set lastone [expr $lastone -1]
+    tkTextSetCursor $t $line.$lastone
+    tkTextInsert $t "\n"
+    CreateEvent "%act: " "dependent" "instantaneous" 1
+}
+
+# Insert Dependent in text editor
+proc InsertHeader {} {
+   global v
+   set t $v(tk,edit)-bis
+
+   set a [$t index insert]
+    regsub {.[0-9]+$} $a "" line 
+    
+    tkTextSetCursor $t "$line.end"
+    set a [$t index insert]
+    regsub {^[0-9]+.} $a "" lineend 
+    set lastone [expr  $lineend- 1]
+    set lastchar [$t get $line.$lastone $line.end]
+    tkTextSetCursor $t $line.$lastone
+    tkTextInsert $t "$lastchar "
+    set lastone [expr $lastone + 2]
+    tkTextSetCursor $t $line.$lastone
+    $t delete insert
+    set lastone [expr $lastone -1]
+    tkTextSetCursor $t $line.$lastone
+    tkTextInsert $t "\n"
+    CreateEvent "@Comment:" "header" "instantaneous" 1
+
+}
+
+# Insert Dependent in text editor
+proc InsertScope {} {
+   global v
+   set t $v(tk,edit)-bis
+
+   CreateEvent "=! " "scope" "instantaneous" 1
+}
