@@ -1,4 +1,6 @@
-# Copyright (C) 1998, DGA - part of the Transcriber program
+# RCS: @(#) $Id: Interface.tcl,v 1.9 2003/08/27 14:55:39 barras Exp $
+
+# Copyright (C) 1998-2000, DGA - part of the Transcriber program
 # distributed under the GNU General Public License (see COPYING file)
 
 ################################################################
@@ -21,13 +23,13 @@ proc LoadImages {} {
 
 #    foreach {name} { } {
 #       set photo [image create photo ${name}Img]
-#       $photo read [glob [file join $v(path,image) $name.gif]]
+#       $photo read [file join $v(path,image) $name.gif]
 #    }
-   foreach {name} { play pause forward backward circle over1 over2 music musicl musicr next previous} {
-      set v(img,$name) [image create bitmap -file \
-			    [glob [file join $v(path,image) $name.bmp]]]
+   foreach {name} { play pause forward backward circle circle2 over1 over2 music musicl musicr next previous} {
+      set v(img,$name) [image create bitmap -file [file join $v(path,image) $name.bmp]]
    }
    $v(img,circle) conf -foreground $v(color,bg-sync)
+   $v(img,circle2) conf -foreground $v(color,bg-sync)
    $v(img,over1) conf -foreground $v(color,bg-sync)
    $v(img,over2) conf -foreground $v(color,bg-sync)
 }
@@ -52,6 +54,9 @@ proc CreateWidgets {} {
    CreateTextFrame .edit
    CreateCommandFrame .cmd
    set v(tk,wavfm) [CreateSoundFrame .snd]
+   if {!$v(view,.edit)} {
+     pack configure .snd -expand true
+   }
    #CreateGainFrame .gain
    if {$v(view,.snd2)} {
       CreateSoundFrame .snd2
@@ -59,6 +64,12 @@ proc CreateWidgets {} {
    CreateMessageFrame .msg
    if {$v(geom,.) != ""} {
       wm geom . $v(geom,.)
+   } else {
+     if {[info tclversion] >= 8.4 && [tk windowingsystem] == "aqua"} {
+       wm geometry . [winfo screenwidth .]x[expr [winfo screenheight .]-44]+0+22 
+     } else {
+       wm geometry . [winfo screenwidth .]x[winfo screenheight .]+0+0 
+     }
    }
 }
 
@@ -199,10 +210,12 @@ proc CreateGainFrame {{f .gain}} {
       toplevel $f
       wm title $f [Local "Control panel"]
 
-      scale $f.s -label [Local "Volume"] \
-	  -orient horiz -length 200 -width 10 \
-	  -variable dial(volume) -command {audio play_gain}
-      pack $f.s -expand true -fill x -padx 10 -pady 5
+     if {[info tclversion] < 8.4 || [tk windowingsystem] != "aqua"} {
+       scale $f.s -label [Local "Volume"] \
+	   -orient horiz -length 200 -width 10 \
+	   -variable dial(volume) -command {snack::audio play_gain}
+       pack $f.s -expand true -fill x -padx 10 -pady 5
+     }
 
       set v(tk,gain)  [scale $f.gain -label [Local "Vertical zoom (dB)"] \
 	       -orient horizontal -length 200 -width 10 -showvalue 1 \
@@ -210,6 +223,7 @@ proc CreateGainFrame {{f .gain}} {
 	       -variable v(sig,gain) -command [list NewGain]]
       pack $f.gain -expand true -fill x -padx 10 -pady 5
 
+      # Disabled, since changing frequency is not yet available
       scale $f.freq -label [Local "Adjust playback speed (%)"] \
 	  -orient horiz -length 200 -width 10 \
 	  -from -40 -to 60 -tickinterval 20 -resolution 1 \
@@ -221,7 +235,7 @@ proc CreateGainFrame {{f .gain}} {
    } else {
       FrontWindow $f
    }
-   set dial(volume) [audio play_gain]
+   set dial(volume) [snack::audio play_gain]
 }
 
 proc AdjustPlaybackSpeed {val} {
@@ -288,8 +302,10 @@ proc SetBindings {} {
    # Some aliases for keyboard events
    bind . <Pause> {PlayOrPause}
    bind . <Alt-Tab> {PlayCurrentSegmt}
-   # Shift-Tab for Linux (Debian1.3) - where does this come from ?
-   catch {bind . <ISO_Left_Tab> {PlayCurrentSegmt}}
+   # alternative for Shift-Tab already defined in menu bindings
+   catch {bind . <Key-ISO_Left_Tab> {PlayCurrentSegmt; break}}
+   # the break added in previous should make following useless:
+   # bind all <<PrevWindow>> {}
 }
 
 #######################################################################
@@ -299,7 +315,7 @@ proc ConfigureGeneral {} {
 
    # Keep initial values for 'Cancel' (but not for 'lang')
    foreach name {
-      scribe,name autosave,time backup,ext
+      scribe,name autosave,time backup,ext encoding
       debug trace,name lang space,auto spell,names
    } {
       lappend initConf $name $v($name)
@@ -318,9 +334,17 @@ proc ConfigureGeneral {} {
    EntryFrame $g.en1 "Auto-save interval (in minutes)" v(autosave,time)
    EntryFrame $g.en2 "Backup extension" v(backup,ext)
 
+   if {![catch {encoding system}]} {
+      EncodingChooser $f
+   }
+
    set g [frame $f.fr3 -relief raised -bd 1]
    pack $g -fill both -expand true -side top
    checkbutton $g.en7 -text [Local "Ask user to save configuration before leaving"] -variable v(keepconfig) -anchor w -padx 3m -pady 2m
+   MenuFrame $g.defpos "Default text cursor position" v(preferedPos) {
+      "Start of line"
+      "End of line"
+   } {"begin" "end"}
    checkbutton $g.en5 -text [Local "Automatic space insertion"] -variable v(space,auto) -anchor w -padx 3m -pady 2m
    checkbutton $g.en6 -text [Local "Check spelling of capitalized words"] -variable v(spell,names) -anchor w -padx 3m -pady 2m
    checkbutton $g.en8 -text [Local "Debug menu"] -variable v(debug) -command {InitMenus} -anchor w -padx 3m -pady 2m
@@ -339,7 +363,7 @@ proc ConfigureGeneral {} {
       if {$lang == $v(lang)} {
 	 set current_nam $nam
       }
-      if {$lang == "en" || [info exists ::local_$lang]} {
+     if {$lang == "en" || [info exists ::local_$lang] || [file readable [file join $v(path,etc) "local_$lang.txt"]]} {
 	 lappend langlist $lang $nam
       } else {
 	 lappend langother $lang $nam
@@ -379,6 +403,54 @@ proc ConfigureGeneral {} {
    }
 }
 
+#######################################################################
+
+# Edit v(encoding) within $f frame - called by ConfigureGeneral
+proc EncodingChooser {f} {
+   global v
+
+   set e [frame $f.enc -relief raised -bd 1]
+   pack $e -fill both -expand true -side top
+
+   label $e.lab -text "[Local Encoding]:"
+   menubutton $e.men -indicatoron 1 -menu $e.men.menu -relief raised -bd 2 -highlightthickness 2 -anchor c
+   menu $e.men.menu -tearoff 0
+   set len 20
+   # List of encoding: IANA name/usual name
+   foreach subl $v(encodingList) {
+      if {$subl == ""} {
+	 $e.men.menu add separator
+	 continue
+      }
+      foreach {val name} $subl {}
+      if {$name == ""} {
+	 set name $val
+      }
+      set name [Local $name]
+      set len [max $len [string length $name]]
+      if {$val == $v(encoding)} {
+	 $e.men configure -text $name
+      }
+      $e.men.menu add radiobutton -label $name -variable v(encoding) -value $val -command [list $e.men configure -text $name]
+   }
+   $e.men configure -width $len
+   pack $e.lab $e.men -side left -padx 3m -pady 3m -fill x -expand true
+}
+
+# Try to find an available Tcl encoding matching the given IANA name
+# and return it, or else an empty string.
+proc EncodingFromName {iana} {
+   set enc [string tolower $iana]
+   regsub "iso-" $enc "iso" enc
+   regsub "_" $enc "" enc
+   # resolve confusion: IANA gb_2312-80 => Tcl gb2312; IANA gb2312 => Tcl euc-cn
+   regsub "gb2312" $enc "euc-cn" enc   
+   if {[lsearch [encoding names] $enc] >= 0} {
+      return $enc
+   } else {
+      return ""
+   }
+}
 
 #######################################################################
 
@@ -565,3 +637,47 @@ proc UpdateColors {} {
       $v(img,over2) conf -foreground $v(color,bg-sync)
    }
 }
+
+proc RandomColor {} {
+  set sum 0
+  while {$sum < 384} {
+    set col "\#"
+    set sum 0
+    foreach c {r g b} {
+      set d [expr int(rand()*256)]
+      append col [format "%02x" $d]
+      incr sum $d
+    }
+  }
+  return $col
+}
+
+proc ColorMap {val {col ""}} {
+  global color
+  regsub -all "\[ _-]" $val "" val
+  set val [string tolower $val]
+  if {$col != ""} {
+    set color($val) $col
+  } elseif {[info exists color($val)]} {
+    set col $color($val)
+  } else {
+    set col [RandomColor]
+    set color($val) $col
+  }
+  return $col
+}
+
+proc ColorizeSpk {{segmt seg1}} {
+  global v color
+  if {$v(colorizeSpk)} {
+    ColorMap ([Local "no speaker"]) $::v(color,bg)
+    for {set i 0} {$i < [GetSegmtNb $segmt]} {incr i} {
+      SetSegmtField $segmt $i -color [ColorMap [GetSegmtField $segmt $i -text]]
+    }
+  } else {
+    catch {unset color}
+    for {set i 0} {$i < [GetSegmtNb $segmt]} {incr i} {
+      SetSegmtField $segmt $i -color ""
+    }
+  }
+}
