@@ -1,4 +1,6 @@
-# Copyright (C) 1998, DGA - part of the Transcriber program
+# RCS: @(#) $Id: Speaker.tcl,v 1.3 2002/07/10 10:04:28 barras Exp $
+
+# Copyright (C) 1998-2000, DGA - part of the Transcriber program
 # distributed under the GNU General Public License (see COPYING file)
 
 ################################################################
@@ -6,6 +8,7 @@
 
 namespace eval speaker {
    variable list_atts {"name" "check" "type" "dialect" "accent" "scope"}
+   variable list_chat {"name" "check" "type" "dialect" "accent" "scope" "role" "age" "birth" "education" "group" "language" "ses" "sex"}
 
    proc init {} {
       variable nb 1
@@ -31,6 +34,14 @@ namespace eval speaker {
       }
    }
 
+   # Replace all references to one speaker with another
+   proc replace {id1 id2} {
+      foreach turn [::xml::dtd::id::get_refs $id1] {
+	 $turn setAttr "speaker" $id2
+	 ::turn::update $turn
+      }      
+   }
+
    # Suppress speaker ids without reference
    proc purge {} {
       variable list
@@ -58,7 +69,8 @@ namespace eval speaker {
 
    # Create a new speaker with the given name inside "Speakers" sub-tree
    # and returns its id, or find an already existing speaker with this name
-   proc create {name {check ""} {type ""} {dialect ""} {accent ""} {scope ""}} {
+   proc create {name {check ""} {type ""} {dialect ""} {accent ""} {scope ""} 
+   {role ""} {age ""} {birth ""} {education ""} {group ""} {language ""} {ses ""} {sex ""} } {
       variable nb
       variable list
 
@@ -87,7 +99,11 @@ namespace eval speaker {
       if {$speakers == ""} {
 	 set speakers [::xml::element "Speakers" {} -begin $v(trans,root)]
       }
-      ::xml::element "Speaker" [list "id" $id "name" $name "check" $check "type" $type "dialect" $dialect "accent" $accent "scope" $scope] -in $speakers
+      if {$::v(chatMode)} {
+	::xml::element "Speaker" [list "id" $id "name" $name "check" $check "type" $type "dialect" $dialect "accent" $accent "scope" $scope "role" $role "age" $age "birth" $birth "education" $education "group" $group "language" $language "ses" $ses "sex" $sex] -in $speakers
+      } else {
+	::xml::element "Speaker" [list "id" $id "name" $name "check" $check "type" $type "dialect" $dialect "accent" $accent "scope" $scope] -in $speakers
+      }
 
       # Keep list of speaker names with associated id
       set list($name) $id
@@ -113,10 +129,16 @@ namespace eval speaker {
 
    # set/returns attributes for given speaker as a list
    proc get_atts {id} {
-      variable list_atts
       set item [::xml::dtd::id::get $id]
-      foreach attr $list_atts {
-	 lappend vals [$item getAttr $attr]
+      if {$::v(chatMode)} {
+	variable list_chat
+	set lst $list_chat
+      } else {
+	variable list_atts
+	set lst $list_atts
+      }
+      foreach attr $lst {
+	lappend vals [$item getAttr $attr]
       }
       return $vals
    }
@@ -145,8 +167,14 @@ namespace eval speaker {
 	 set upd 0
       }
       # Really change attributes
-      variable list_atts
-      foreach attr $list_atts val $vals {
+      if {$::v(chatMode)} {
+	variable list_chat
+	set lst $list_chat
+      } else {
+	variable list_atts
+	set lst $list_atts
+      }
+      foreach attr $lst val $vals {
 	 $item setAttr $attr $val
       }
       # Update new name in editor and segmentation after changes
@@ -223,6 +251,7 @@ namespace eval speaker {
    # Sorted list of all registred names
    proc find_names {} {
       variable list
+      set spklst {}
       foreach {name id} [array get list] {
 	 set l [concat $id [get_atts $id]]
 	 lappend spklst $l
@@ -424,6 +453,16 @@ namespace eval turn {
       variable acc
       variable ::speaker::recent
 
+# added by Zhibiao
+      variable role
+      variable age 
+      variable birth
+      variable education
+      variable group
+      variable language
+      variable ses 
+      variable sex 
+      
       # get speaker list and overlap state
       set ids [lindex $atts 0]
       set id1 [lindex $ids $pos]
@@ -442,6 +481,7 @@ namespace eval turn {
       }
       set cha [lindex $atts 3]
       #
+      set pad 1m
       set w [CreateModal .turn "Edit turn attributes"]
       #
       set w3 [frame $w.top -relief raised -bd 1]
@@ -449,7 +489,7 @@ namespace eval turn {
       checkbutton $w3.0 -text [Local "Overlapping speech"] -variable ::turn::ovl -command "::turn::choose_overlap"
       radiobutton $w3.1 -text [Local "Choose first speaker"] -variable ::turn::newpos -value 0 -command "::turn::choose_register; ::turn::choose_switch"
       radiobutton $w3.2 -text [Local "Choose second speaker"] -variable ::turn::newpos -value 1 -command "::turn::choose_register; ::turn::choose_switch"
-      pack $w3.0 $w3.1 $w3.2 -side left -anchor w -padx 3m -pady 3m -expand true
+      pack $w3.0 $w3.1 $w3.2 -side left -anchor w -padx $pad -pady $pad -expand true
       #
       set w0 [frame $w.mid]
       pack $w0 -side top -fill both -expand true
@@ -463,9 +503,9 @@ namespace eval turn {
       set w11 [frame $w1.up]
       pack $w11 -side top -fill both -expand 1
       button $w11.new -text [Local "Create speaker"] -command {::turn::choose_new}
-      pack $w11.new -side left -padx 3m -pady 3m -expand true
+      pack $w11.new -side left -padx $pad -pady $pad -expand true
       variable edit [button $w11.edit -text [Local "Modify speaker"] -command {::turn::choose_edit}]
-      pack $w11.edit -side left -padx 3m -pady 3m -expand true
+      pack $w11.edit -side left -padx $pad -pady $pad -expand true
       #
       set w12 [frame $w1.spkatt]
       variable watt $w12
@@ -475,28 +515,42 @@ namespace eval turn {
       pack $w12.bts -side top -expand true -anchor w
       checkbutton $w12.bts.chk -text [Local "spelling checked"] -var ::turn::chk -onvalue "yes" -offvalue "no" -state disabled
       checkbutton $w12.bts.exp -text [Local "global name"] -var ::turn::exp -onvalue "global" -offvalue "local" -state disabled
-      pack $w12.bts.chk $w12.bts.exp -side left -padx 3m -pady 3m -expand true -anchor w
+      pack $w12.bts.chk $w12.bts.exp -side left -padx $pad -pady $pad -expand true -anchor w
       MenuFrame $w12.typ "Type" ::turn::typ {"male" "female" "unknown"}
       MenuFrame $w12.dia "Dialect" ::turn::dia {"native" "nonnative"}
+
+     if {$::v(chatMode)} {
+#### added by Zhibiao
+      EntryFrame $w12.role "CH_Role" ::turn::role
+      EntryFrame $w12.age  "CH_Age" ::turn::age 
+      EntryFrame $w12.birth "CH_Birth" ::turn::birth
+      EntryFrame $w12.education "CH_Education" ::turn::education
+      EntryFrame $w12.group "CH_Group" ::turn::group
+      EntryFrame $w12.langauge "CH_Language" ::turn::language
+      EntryFrame $w12.ses "CH_Ses" ::turn::ses
+      EntryFrame $w12.sex "CH_Sex" ::turn::sex
+# added end
+    }
+
       entry $w12.dia.acc -text ::turn::acc -width 15
-      pack $w12.dia.acc -expand true -side left -padx 3m -pady 3m
+      pack $w12.dia.acc -expand true -side left -padx $pad -pady $pad
       #EntryFrame $w12.acc "Accent" ::turn::acc
       #
       set w21 [frame $w2.up]
       pack $w21 -side top -fill both
       #label $w2.lab -text [Local "Choose speaker"]
-      #pack $w2.lab -side top -padx 3m -pady 3m -expand true
+      #pack $w2.lab -side top -padx $pad -pady $pad -expand true
       set w211 [frame $w21.mid]
       pack $w211 -side top -fill x
       #if {[llength $ids] <= 1} 
       radiobutton $w211.none -text [Local "no speaker"] -variable ::turn::nam -value "" -command {::turn::choose_empty}
-      pack $w211.none -side left -padx 3m -pady 3m
+      pack $w211.none -side left -padx $pad -pady $pad
       if {[llength $recent] > 0} {
 	 label $w21.lab -text [Local "Recently used speakers"]:
-	 pack $w21.lab -side top -padx 3m -anchor w
+	 pack $w21.lab -side top -padx $pad -anchor w
 	 foreach i [lrange $recent 0 2] {
 	    radiobutton $w21.$i -text [string range [::speaker::name $i] 0 20] -variable ::turn::id1 -value $i -command "::turn::choose_recent $i"
-	    pack $w21.$i -side top -anchor w -padx 3m
+	    pack $w21.$i -side top -anchor w -padx $pad
 	 }
       }
       #
@@ -505,6 +559,7 @@ namespace eval turn {
       variable lst [ListFrame $w22.lst [::speaker::all_names]]
       $lst conf -height 6
       bind $lst <ButtonRelease-1>  "::turn::choose_lst"
+      bind $lst <Key-space>  "::turn::choose_lst"
       # Detect normal keypress when entry is disabled
       foreach key {Control-Key Alt-Key Meta-Key Return Escape Tab} {
 	 bind $ent <$key> {continue}
@@ -560,10 +615,37 @@ namespace eval turn {
       variable ovl
       variable state
 
+# added by Zhibiao
+      variable role
+      variable age 
+      variable birth
+      variable education
+      variable group
+      variable language
+      variable ses 
+      variable sex 
+
       if {$state == "new"} {
-	 set id1 [::speaker::create $nam $chk $typ $dia $acc $exp]
+	if {$::v(chatMode)} {
+	  set id1 [::speaker::create $nam $chk $typ $dia $acc $exp $role $age $birth $education $group $language $ses $sex]
+	} else {
+	  set id1 [::speaker::create $nam $chk $typ $dia $acc $exp]
+	}
       } elseif {$state == "edit"} {
-	 ::speaker::set_atts $id1 [list $nam $chk $typ $dia $acc $exp]
+	 set id [::speaker::search_id $nam]
+	 if {$id != "" && $id != $id1} {
+	    set answer [tk_messageBox -message "Speaker $nam already exists. Replace [::speaker::name $id1] with $nam everywhere?" -type yesno -icon question]
+	    if {$answer == "yes"} {
+	       ::speaker::replace $id1 $id
+	    }
+	    set id1 $id
+	 } else {
+	   if {$::v(chatMode)} {
+	     ::speaker::set_atts $id1 [list $nam $chk $typ $dia $acc $exp $role $age $birth $education $group $language $ses $sex]
+	   } else {
+	     ::speaker::set_atts $id1 [list $nam $chk $typ $dia $acc $exp]
+	   }
+	 }
       }
       if {$id1 == ""} {
 	 if {$ovl} {
@@ -628,6 +710,17 @@ namespace eval turn {
 
       variable state "new"
       variable edit
+
+# added by Zhibiao
+      variable role
+      variable age 
+      variable birth
+      variable education
+      variable group
+      variable language
+      variable ses 
+      variable sex 
+
       $edit conf -state disabled
 
       variable lst
@@ -684,8 +777,23 @@ namespace eval turn {
       variable state "choose"
       variable edit
 
+# added by zhibiao
+      variable role ""
+      variable age 
+      variable birth
+      variable education
+      variable group
+      variable language
+      variable ses 
+      variable sex 
+# added end
+
       if {$id != ""} {
-	 foreach {nam chk typ dia acc exp} [::speaker::get_atts $id] {}
+	if {$::v(chatMode)} {
+	  foreach {nam chk typ dia acc exp role age birth education group language ses sex} [::speaker::get_atts $id] {}
+	} else {
+	  foreach {nam chk typ dia acc exp} [::speaker::get_atts $id] {}
+	}
 	 $edit conf -state normal
       } else {
 	 $edit conf -state disabled
