@@ -1,6 +1,95 @@
-# Copyright (C) 1998, DGA - part of the Transcriber program
+# RCS: @(#) $Id: Trans.tcl,v 1.10 2003/08/28 13:55:50 barras Exp $
+
+# Copyright (C) 1998-2000, DGA - part of the Transcriber program
 # distributed under the GNU General Public License (see COPYING file)
 
+################################################################
+# Native transcription in XML format I/O
+
+namespace eval trs {
+   variable msg "Native XML format"
+   variable ext ".trs .xml"
+
+   proc import {name} {
+      global v
+      
+      # Newly created transcription must follow the DTD
+      ::xml::dtd::xml_read $v(file,dtd)
+      set v(trans,root) [::xml::parser::read_file $name -keepdtd 1]
+      ::speaker::register
+      ::topic::register
+   }
+
+   proc export {name} {
+      global v
+      
+      ::xml::parser::write_file $name $v(trans,root)
+   }
+}
+
+################################################################
+# Init handlers for all formats
+
+proc InitConvertors {} {
+   global v
+
+   foreach module [glob [file join $v(path,base) "convert" *.tcl]] {
+      namespace eval convert [list source $module]
+   }
+
+   set v(ext,trs) {}
+   set v(ext,lbl) {}
+   set v(newtypes) {}
+   foreach format [concat "trs" [namespace children convert]] {
+      upvar 0 ${format}::msg msg
+      upvar 0 ${format}::ext ext
+      if {[info command ${format}::import] != ""
+	  || [info command ${format}::readSegmt] != ""
+	  || [info command ${format}::readSegmtSet] != ""} {
+	 lappend v(newtypes) [list $msg $ext]
+	 # Add default guess proc from input extensions
+	 if {[info command ${format}::guess] == ""} {
+	    namespace eval $format {
+	       proc guess {name} {
+		  variable ext
+		  set filext [string tolower [file extension $name]]
+		  return [expr [lsearch $ext $filext]>=0]
+	       }
+	    }
+	 }
+	 # Add default readSegmt proc from readSegmtSet
+	 if {[info command ${format}::readSegmt] == ""
+	     && [info command ${format}::readSegmtSet] != ""} {
+	   namespace eval $format {
+	     proc readSegmt {content} {
+	       return [lindex [lindex [readSegmtSet $content] 0] 0]
+	     }
+	   }
+	 }
+	 if {[info command ${format}::import] != ""} {
+	    eval lappend v(ext,trs) $ext
+	 } elseif {[info command ${format}::readSegmt] != ""} {
+	    eval lappend v(ext,lbl) $ext
+	 }
+      }
+   }
+   #UpdateConvertorMenu
+}
+
+proc UpdateConvertorMenu {} {
+   global v
+
+   foreach format [namespace children convert] {
+      upvar 0 ${format}::msg msg
+      if {[info command ${format}::export] != ""} {
+	 if {[namespace tail $format] == "cha" && !$v(chatMode)} continue
+	 append_menu "Export" [subst {
+	    {"Export to $msg..."	cmd {SaveTrans as $format}}
+	 }]
+      }
+   }
+}
+
 ################################################################
 # Read single-level segmentations
 
@@ -10,124 +99,79 @@ proc ReadFile {fileName} {
 
    set channel [open $fileName r]
    # Read with chosen encoding (intended for tcl/tk 8.1 only)
-   if {[info exists v(encoding)] && $v(encoding) != ""} {
-      fconfigure $channel -encoding $v(encoding)
+   if {![catch {encoding system}]} {
+      fconfigure $channel -encoding [EncodingFromName $v(encoding)]
    }
    set text [read -nonewline $channel]
    close $channel
    return $text
 }
 
-
-# Read TIMIT format
-proc ReadSegmtTimit {content} {
-   set segmt {}
-   foreach line [string trim [split $content "\n"]] {
-      if {$line == ""} continue
-      set begin [expr [lindex $line 0]/16000.0]
-      set end   [expr [lindex $line 1]/16000.0]
-      set text  [lrange $line 2 end]
-      lappend segmt [list $begin $end $text]
+proc LookForLabelFormat {name} {
+   global v
+   # Look for a segmentation format matching the file
+   set format ""
+  foreach ns [lsort [namespace children convert]] {
+     if {[info command ${ns}::readSegmt] != "" && [${ns}::guess $name]} {
+       set format $ns
+       break
+     }
    }
-   return $segmt
-}
-
-# Read OGI Lola format
-proc ReadSegmtLola {content} {
-   set mpf 10.0
-   set header 1
-   set segmt {}
-   foreach line [string trim [split $content "\n"]] {
-      if {$line == ""} continue
-      if {$header} {
-	 switch -glob -- $line {
-	    "MillisecondsPerFrame:*" {
-	       set mpf [lindex $line 1] }
-	    "END OF HEADER" {
-	       set header 0
-	    }
-	 }
-      } else {
-	 set begin [expr [lindex $line 0]*$mpf/1000.0]
-	 set end   [expr [lindex $line 1]*$mpf/1000.0]
-	 set text  [lrange $line 2 end]
-	 lappend segmt [list $begin $end $text]
-      }
-   }
-   return $segmt
-}
-
-# Read xwaves label format
-proc ReadSegmtLab {content} {
-   set end 0.0
-   set header 1
-   set segmt {}
-   foreach line [string trim [split $content "\n"]] {
-      if {$line == ""} continue
-      if {$header} {
-	 switch -glob -- $line {
-	    "\#" {
-	       set header 0
-	    }
-	 }
-      } else {
-	 set begin $end
-	 set end   [lindex $line 0]
-	 set text  [lrange $line 2 end]
-	 lappend segmt [list $begin $end $text]
-      }
-   }
-   return $segmt
-}
-
-# Read Limsi LBL
-proc ReadSegmtLbl {content} {
-   set segmt {}
-   set begin 0.0
-   foreach line [string trim [split $content "\n"]] {
-      if {$line == ""} continue
-      set end   [lindex $line 0]
-      if {$end > $begin} {
-	 lappend segmt [list $begin $end $text]
-      }
-      set text  [lrange $line 1 end]
-      set begin $end
-   }
-   return $segmt
+   return $format
 }
 
 # Open a new independant segmentation file
-# This function is not yet interfaced (has to be called manually)
-proc OpenSegmt {} {
+proc OpenSegmt {{name ""}} {
    global v
 
-   # Choose one empty seg
-   set i 3
-   while {[info exists v(trans,seg$i)]} {
-      incr i
+   if {$name == ""} {
+     set types [subst {
+       {"Label file"   {$v(ext,lbl)}}
+       {"All files"   {*}}
+     }]
+     if {$v(labelNames) != {}} {
+       set path [file dir [lindex $v(labelNames) 0]]
+     } elseif {$v(trans,name) == "" && $v(sig,name) != ""} {
+       set path [file dir $v(sig,name)]
+     } else {
+       set path $v(trans,path)
+     }
+     set name [tk_getOpenFile -filetypes $types -initialdir $path \
+		   -title [Local "Open segmentation file"]]
    }
-   set seg "seg$i"
-
-   set types [subst {
-      {"Label file"   {$v(ext,lbl)}}
-      {"All files"   {*}}
-   }]
-   set name [tk_getOpenFile -filetypes $types -initialdir $v(trans,path) \
-		 -title [Local "Open segmentation file"]]
    if {$name != ""} {
-      set format [string trimleft [file extension $name] "."]
-      switch -exact -- $format {
-	 "lab" -
-	 "esps"  { set v(trans,$seg) [ReadSegmtLab [ReadFile $name]] }
-	 "lbl"  { set v(trans,$seg) [ReadSegmtLbl [ReadFile $name]] }
-	 "lola"  { set v(trans,$seg) [ReadSegmtLola [ReadFile $name]] }
-	 "phn" - "wrd" - "txt" -
-	 "timit" { set v(trans,$seg) [ReadSegmtTimit [ReadFile $name]] }
-	 default { error [concat [Local "Unknown format"] $format] }
+      if {![file readable $name]} return
+      # Look for a segmentation format matching the file
+      set format [LookForLabelFormat $name]
+      if {$format == ""} {
+	 error [format [Local "Unknown format for file %s"] $name]
       }
-      foreach wavfm $v(wavfm,list) {
-	 CreateSegmentWidget $wavfm $seg -fg $v(color,fg-sync) -full $v(color,bg-sync)
+      # Choose uique segmentation id
+      set i 1
+      set seg lbl$i
+      if {[info command ${format}::readSegmtSet] != ""} {
+	set result [${format}::readSegmtSet [ReadFile $name]]
+      } else {
+	set result [list [list [${format}::readSegmt [ReadFile $name]]]]
       }
+      foreach set $result {
+	foreach {segments entryname view color} $set break
+	if {$view == ""} {
+	  set view 1
+	}
+	if {$color == ""} {
+	  set color white
+	}
+	while {[info exists v(trans,$seg)]} {
+	  set seg lbl[incr i]
+	}
+	set v(trans,$seg) $segments
+	foreach wavfm $v(wavfm,list) {
+	  set v(view,[winfo parent $wavfm].$seg) $view
+	  CreateSegmentWidget $wavfm $seg "[file tail $name] $entryname" -full $color
+	}
+      }
+      lappend v(labelNames) $name
    }
 }
 
@@ -136,7 +180,7 @@ proc SegmtToTrans {segmt} {
    global v
 
    # Newly created transcription must follow the DTD
-   ::xml::dtd::read $v(file,dtd)
+   ::xml::dtd::xml_read $v(file,dtd)
 
    if {$v(sig,name) != ""} {
       set t0 $v(sig,min)
@@ -168,171 +212,46 @@ proc SegmtToTrans {segmt} {
 ################################################################
 # Read transcriptions
 
-# Read LDC typ (added field for topic, <bg> for Background)
-proc ReadTransTyp {content} {
-   global v
-
-   array set typ2ML {"sn" "nontrans" "sr" "report" "sf" "filler"}
-
-   ::xml::dtd::read $v(file,dtd)
-   set v(trans,root) [::xml::element "Trans"]
-#    if {[info exists v(file,speakers)]} {
-#       $v(trans,root) addChilds [::xml::parser::read_file $v(file,speakers) -dtdname $v(file,dtd)]
-#       ::speaker::register
-#    }
-   set episode [::xml::element "Episode" {} -in $v(trans,root)]
-
-   # Transcription begins implicitly with a "nontrans" section
-   set begin 0
-   set text ""
-   set type "sn"
-   set topic ""
-   set speaker ""
-   set sec ""
-   set tur ""
-   set lines [split $content "\n"]
-   if {$v(sig,name) != ""} {
-      set lines [concat $lines [list "<sn $v(sig,max)>"]]
-   }
-   foreach line $lines {
-      if [regexp {<([a-z][a-z0-9]*) ([0-9.]+)( ([^>]+))?>( <<((male|female|child), ((I|O), )?)?(.*)>>)?} \
-	      $line match code time hasattrib attrib hasname hasgender gender hasnativ nativ name] {
-	 if {[info exists begin] && $time>0} {
-	    if {$time<$begin} {
-	       continue
-	       #error "Segments are not in right order ($time<$begin)" 
-	    } elseif {$time == $begin} {
-	       # This can happen for near values with the 1ms precision
-	       #continue
-	    }
-	    if {![catch {set t $typ2ML($type)}]} {
-	       if {$tur != ""} {
-		  $tur setAttr "endTime" $begin
-	       }
-	       set tur ""
-	       if {$sec != ""} {
-		  $sec setAttr "endTime" $begin
-	       }
-	       set sec [::xml::element "Section" "type $t" -in $episode]
-	       if {$topic != ""} {
-		  $sec setAttr "topic" $topic
-	       }
-	       $sec setAttr "startTime" $begin
-	    } else {
-	       set t ""
-	    }
-	    if {$speaker != "" || $t != "" || $type == "e1" || $type == "e2"} {
-	       if {$type == "e2"} {
-		  set speaker [lindex $overspk 0]
-	       } elseif {$type == "e1"} {
-		  set speaker [lindex $overspk 1]
-	       }
-	       if {$tur != ""} {
-		  $tur setAttr "endTime" $begin
-		  if {$type == "o"} {
-		     set speaker [concat [$tur getAttr "speaker"] $speaker]
-		     set overspk $speaker
-		     #regexp ".*SPEAKER1: ?(.*) SPEAKER2: ?(.*)" $text all t1 t2
-		     #set text "\[1] $t1 \[2] $t2"
-		  }
-	       }
-	       set tur [::xml::element "Turn" {} -in $sec]
-	       if {$speaker != ""} {
-		  $tur setAttr "speaker" $speaker
-	       }
-	       $tur setAttr "startTime" $begin
-	    }
-	    if {$type == "bg"} {
-	       set m [expr [llength $bg]-1]
-	       set bgTyp [lrange $bg 0 [expr $m-1]]
-	       set bgLvl [lrange $bg $m end]
-	       set attrs [list "time" $begin "type" $bgTyp "level" $bgLvl]
-	       set sync [::xml::element "Background" $attrs -in $tur]
-	    } else {
-	       set sync [::xml::element "Sync" "time $begin" -in $tur]
-	    }
-	    if {$time > $begin} {
-	       if {$text != ""} {::xml::data $text -in $tur}
-	    }
-	 } else {
-	    # Just in case first line is not marked as a section
-	    if {[lsearch [array names typ2ML] $code] < 0} {
-	       set code "sr"
-	    }
-	 }
-	 set begin $time
-	 set type  $code
-	 if {$type != "bg"} {
-	    set topic [::topic::create $attrib]
-	 } else {
-	    set bg [string tolower $attrib]
-	    set topic ""
-	 }
-	 regsub -all "_" $name " " name
-	 set speaker ""
-	 foreach onename [split $name "+"] {
-	    set onename [string trim $onename]
-	    lappend speaker [::speaker::create $onename "" $gender]
-	 }
-	 set text ""
-      } else {
-	 if {$text != ""} {
-	    append text " "
-	 }
-	 append text $line
-      }
-   }
-   if [info exists begin] {
-      if {$tur != ""} {
-   	 $tur setAttr "endTime" $begin
-      }
-      if {$sec != ""} {
-   	 $sec setAttr "endTime" $begin
-      }
-   }
-}
-
-# Transcription in XML format
-proc ReadTransXML {name} {
-   global v
-
-   # Newly created transcription must follow the DTD
-   ::xml::dtd::read $v(file,dtd)
-   set v(trans,root) [::xml::parser::read_file $name -keepdtd 1]
-   ::speaker::register
-   ::topic::register
-}
-
 # Read transcription file in several formats
-# Called from: OpenTransFile, RevertTrans, StartWith
-proc ReadTrans {name {soundFile ""} {format ""}} {
+# Called from: OpenTrans(OrSound)File, RevertTrans, StartWith
+proc ReadTrans {name {soundFile ""} {multiwav {}}} {
    global v
    
    # First, try to rescue from last autosaved file if it exists.
    AutoRescue $name
 
-   if {$format == ""} {
-      set format [string trimleft [file extension $name] "."]
+   set format "trs"
+   foreach ns [namespace children convert] {
+      if {[info command ${ns}::guess] != "" && [${ns}::guess $name]} {
+	 set format $ns
+	 break
+      }
    }
+   #puts "format $format"
+
    DisplayMessage [Local "Cleaning up memory..."]; update
    CloseTrans -nosave
 
    # Try to open associated sound file as early as possible for non-xml files
-   if {$format != "xml" && $format !="trs"} {
+   if {$format !="trs"} {
       LookForSignal $name $soundFile
    }
 
    DisplayMessage [Local "Reading transcription file..."]; update
-   switch -exact -- $format {
-      "typ"   { ReadTransTyp [ReadFile $name] }
-      "lab" -
-      "esps"  { SegmtToTrans [ReadSegmtLab [ReadFile $name]] }
-      "lbl"   { SegmtToTrans [ReadSegmtLbl [ReadFile $name]] }
-      "lola"  { SegmtToTrans [ReadSegmtLola [ReadFile $name]] }
-      "phn" - "wrd" - "txt" -
-      "timit" { SegmtToTrans [ReadSegmtTimit [ReadFile $name]] }
-      default { ReadTransXML $name; set format "trs" }
+   if {[info command ${format}::import] != ""} {
+      ${format}::import $name
+   } elseif {[info command ${format}::readSegmt] != ""} {
+      SegmtToTrans [${format}::readSegmt [ReadFile $name]]
+   } else {
+      error "Can't import from [set ${format}::msg]"
    }
+
+   if {[namespace tail $format] == "cha"} {
+     setChatMode 1
+   } else {
+     setChatMode 0
+   }
+
    set v(trans,name) $name
    UpdateShortName
    set v(trans,format) $format
@@ -345,20 +264,41 @@ proc ReadTrans {name {soundFile ""} {format ""}} {
    TraceOpen
 
    # Try to open automatically sound file else ask user
-   if {$format == "xml" || $format == "trs"} {
-      LookForSignal $name $soundFile
+   if {$format == "trs"} {
+      LookForSignal $name $soundFile [lindex [GetFilename] 0]
    } else {
       # For newly created transcriptions, keep info about signal basename
       UpdateFilename
    }
    if {$v(sig,name) == ""} {
-      tk_messageBox -type ok -icon warning -message \
-	  [concat [Local "Please open signal for transcription"] $name]
-      OpenAudioFile
+      set rep [tk_messageBox -type okcancel -icon warning -message \
+	  [concat [Local "Please open signal for transcription"] $name]]
+      if {$rep == "ok"} {
+	OpenAudioFile
+      }
       if {$v(sig,name) == ""} {
 	 EmptySignal
       }
    }
+   # add list of sound files found in .trs header
+   if {$format == "trs"} {
+     MW_Update
+   }
+   # also add multiple wav files from command line or configuration
+   if {[llength $multiwav] > 1} {
+     eval MW_AddFile $multiwav
+     UpdateFilename
+   }
+}
+
+proc setChatMode {newMode} {
+  global v
+
+  set oldMode $v(chatMode)
+  set v(chatMode) $newMode
+  if {$v(chatMode) != $oldMode} {
+    InitMenus
+  }
 }
 
 # Open transcription file through selection box
@@ -369,14 +309,8 @@ proc OpenTransFile {} {
    set types [subst {
       {"Transcription"   {$v(ext,trs)}}
       {"Label file"   {$v(ext,lbl)}}
-      {"XML format" {.xml .trs}}
-      {"LDC format" {.typ}}
-      {"ESPS/xwaves" {.lab}}
-      {"OGI lola" {.lola}}
-      {"Limsi label" {.lbl}}
-      {"TIMIT" {.phn .wrd .txt}}
-      {"All files"   {*}}
    }]
+   eval lappend types $v(newtypes) {{"All files"   {*}}}
    set name [tk_getOpenFile -filetypes $types -initialdir $v(trans,path) \
 		 -title [Local "Open transcription file"]]
    if {$name != ""} {
@@ -425,272 +359,29 @@ proc OpenTransOrSoundFile {} {
 ################################################################
 # Write transcriptions
 
-# By default, extend typ format with topic and background infos
-# Do not convert overlapping speech to standard typ format  <o> <e.>
-# but as: <t> <<speaker A + speaker B>> \n [1] ... [2] ...
-proc WriteTransTyp {name {extend 1}} {
-   global v
-   array set ML2typ {"nontrans" "sn" "report" "sr" "filler" "sf"}
-
-   set topic ""
-   set channel [open $name w]
-   set episode [$v(trans,root) getChilds "element" "Episode"]
-   foreach sec [$episode getChilds "element" "Section"] {
-      set type $ML2typ([$sec getAttr "type"])
-      set time [$sec getAttr "startTime"]
-      if {$extend} {
-	 set topic [$sec getAttr topic]
-	 if {$topic != ""} {
-	    set topic " [::topic::get_atts $topic]"
-	 }
-      }
-      if {$time > 0 || $type != "sn" || $extend} {
-	 puts -nonewline $channel [format "<$type %.3f$topic>" $time]
-      }
-      set turns [$sec getChilds "element" "Turn"]
-      for {set nt 0} {$nt < [llength $turns]} {incr nt} {
-	 set tur [lindex $turns $nt]
-	 set spk [$tur getAttr "speaker"]
-	 if {$spk != ""} {
-	    set spk [::speaker::name $spk]
-	 }
-	 if {[string index $type 0] != "s"} {
-	    set time [$tur getAttr "startTime"]
-	    puts -nonewline $channel [format "<t %.3f>" $time]
-	 }
-	 if {$spk != ""} {
-	    puts $channel " <<$spk>>"
-	 } elseif {$time > 0 || $type != "sn" || $extend} {
-	    puts $channel ""
-	 }
-	 set type "t"
-	 set do_nl 0
-	 foreach chn [$tur getChilds] {
-	    if {[$chn class] == "data"} {
-	       set text [$chn getData]
-	       if {$text != ""} {
-		  puts -nonewline $channel $text
-		  set do_nl 1
-	       }
-	    } elseif {[$chn class] == "element"} {
-	       switch [$chn getType] {
-	       "Sync" {
-		  if {$type != "t"} {
-		     set time [$chn getAttr "time"]
-		     if {$do_nl} {puts $channel ""; set do_nl 0}
-		     puts $channel [format "<b %.3f>" $time]
-		  }
-		  set type "b"
-	       }
-	       "Background" {
-		  set time [$chn getAttr "time"]
-		  set bgTyp [$chn getAttr "type"]
-		  set bgLvl [$chn getAttr "level"]
-		  # Background saved as extension of typ format
-		  if {$extend} {
-		     if {$do_nl} {puts $channel ""; set do_nl 0}
-		     puts $channel [format "<bg %.3f $bgTyp $bgLvl>" $time]
-		  }
-		  set type "b"
-	       }
-	       "Who" {
-		  set nb [$chn getAttr "nb"]
-		  if {$nb > 1} { puts $channel "" }
-		  puts -nonewline $channel "SPEAKER$nb: "
-		  set do_nl 1
-	       }
-	       "Event" - "Comment" {
-		  puts -nonewline $channel [StringOfEvent $chn]
-		  set do_nl 1
-	       }
-	       }
-	    }
-	 }
-	 if {$do_nl} {puts $channel ""; set do_nl 0}
-      }
-   }
-   close $channel
-}
-
-proc WriteTransSTM {name} {
+# Write transcription to the file in specified format; returns chosen format.
+proc WriteTrans {name format} {
    global v
 
-   #;; LABEL "F0" "Baseline//Broadcast//Speech" ""
-   #;; LABEL "F1" "Spontaneous//Broadcast//Speech" ""
-   #;; LABEL "F2" "Speech Over//Telephone//Channels" ""
-   #;; LABEL "F3" "Speech in the//Presence of//Background Music" ""
-   #;; LABEL "F4" "Speech Under//Degraded//Acoustic Conditions" ""
-   #;; LABEL "F5" "Speech from//Non-Native//Speakers" ""
-   #;; LABEL "FX" "All other speech" ""
-
-   set head ""
-   set time ""
-   set bgLvl "off"
-   set bgTyp ""
-   set nxt ""
-   set channel [open $name w]
-   #set base [$v(trans,root) getAttr "audio_filename"]
-   set base [file root [file tail $name]]
-   set episode [$v(trans,root) getChilds "element" "Episode"]
-   foreach sec [$episode getChilds "element" "Section"] {
-      foreach tur [$sec getChilds "element" "Turn"] {
-	 set turncond "f0"
-	 if {[$tur getAttr "mode"] == "spontaneous"} {
-	    set turncond "f1"
-	 }
-	 if {[$tur getAttr "channel"] == "telephone"} {
-	    set turncond "f2"
-	 }
-	 if {[$tur getAttr "fidelity"] == "low"} {
-	    set turncond "f4"
-	 }
-	 set spk [$tur getAttr "speaker"]
-	 set gender ""
-	 set scope "global"
-	 if {$spk != ""} {
-	    if {[llength $spk] == 1} {
-	       catch {
-		  set atts [::speaker::get_atts $spk]
-		  set gender [lindex $atts 2]
-		  set scope [lindex $atts 5]
-		  if {[lindex $atts 3] == "nonnative"} {
-		     if {$turncond == "f4"} {
-			set turncond "fx"
-		     } else {
-			set turncond "f5"
-		     }
-		  }
-	       }
-	    } else {
-	       set turncond "f4"
-	    }
-	    set spk [::speaker::name $spk]
-	    if {$scope != "global"} {
-	       set spk "$base $spk"
-	    }
-	 }
-	 foreach chn [$tur getChilds] {
-	    if {[$chn class] == "data"} {
-	       set data [$chn getData]
-	       if {$nxt != ""} {
-		  if {[regexp { *([^ ]+)( .*)} $data all wrd data]} {
-		     append txt [format $nxt $wrd]
-		  }
-		  set nxt ""
-	       }
-	       append txt $data
-	    } elseif {[$chn class] == "element"} {
-	       switch [$chn getType] {
-	       "Sync" - "Background" {
-		  set newtime [format %.3f [$chn getAttr "time"]]
-		  catch  {
-		     set bgTyp [$chn getAttr "type"]
-		     set bgLvl [$chn getAttr "level"]
-		  }
-		  if {$bgLvl == "off"} {
-		     set cond $turncond
-		  } elseif {$bgTyp == "music"} {
-		     if {$turncond == "f4" || $turncond == "f5"} {
-			set cond "fx"
-		     } else {
-			set cond "f3"
-		     }
-		  } else {
-		     if {$turncond == "f5"} {
-			set cond "fx"
-		     } else {
-			set cond "f4"
-		     }
-		  }
-		  if {$newtime > $time || $time == ""} {
-		     set time $newtime
-		     if {$head != ""} {
-			puts $channel "$head $time $txt"
-		     }
-		  }
-		  set head "$base 1 \"$spk\" $time"
-		  set txt "<o,$cond,$gender> "
-	       }
-	       "Who" {
-		  set nb [$chn getAttr "nb"]
-		  append txt " \[$nb] "
-	       }
-	       "Comment" {
-		  set desc [$chn getAttr "desc"]
-		  append txt "<comment>$desc</comment>"
-	       }
-	       "Event" {
-		  set desc [$chn getAttr "desc"]
-		  set type [$chn getAttr "type"]
-		  set extn [$chn getAttr "extent"]
-		  if {$type == "noise"} {
-		     set f(begin) "\[$desc-]"
-		     set f(end) "\[-$desc]"
-		     set f(instantaneous) "\[$desc]"
-		  } else {
-		     if {$type == "language"} {
-			catch {set desc $::iso639($desc)}
-		     }
-		     set f(begin) "<$type=$desc>"
-		     set f(end) "</$type>"
-		     set f(instantaneous) "$f(begin) $f(end)"
-		  }		     
-		  switch $extn {
-		     "previous" {
-			if {[regexp {(.* )([^ ]+) *} $txt all txt prv]} {
-			   append txt "$f(begin) $prv $f(end)"
-			}
-		     }
-		     "next" {
-			set nxt "$f(begin) %s $f(end)"
-		     }
-		     "begin" - "end" - "instantaneous" {
-			append txt $f($extn)
-		     }
-		  }
-	       }
-	       }
-	    }
-	 }
+   set ext [string trimright [string tolower [file extension $name]] "\#"]
+   # Try to guess format from extension if not given - unused feature
+#    if {$format == ""} {
+#       foreach format [concat [namespace children convert] "trs"] {
+# 	 if {[info exists ${format}::ext]
+# 	     && [lsearch [set ${format}::ext] $ext] >= 0} {
+# 	    break
+# 	 }
+#       }
+#    }
+   if {[lsearch [set ${format}::ext] $ext] < 0} {
+      if {[tk_messageBox -type yesno -icon question -message [format [Local "%s is not a standard extension for %s. Continue?"] $ext [set ${format}::msg]]] != "yes"} {
+	 error "" 
       }
    }
-   set time [format %.3f [$tur getAttr "endTime"]]
-   if {$head != ""} {
-      puts $channel "$head $time $txt"
-   }
-   close $channel
-}
-
-proc WriteTransXML {name} {
-   global v
-
-   ::xml::parser::write_file $name $v(trans,root)
-}
-
-proc WriteTransOther {name} {
-   tk_messageBox -type ok -icon error -message \
-       [concat $name [Local "not saved (unsupported output format)"]]
-}
-
-# Write transcription to the file in specified format (or try to guess
-# it from name extension); returns chosen format.
-proc WriteTrans {name {format ""}} {
-   global v
-
-   if {$format == ""} {
-      set format [string trimleft [file extension $name] "."]
-   }
-   switch -exact -- $format {
-      "trs"  -
-      "xml"  { WriteTransXML $name }      
-      "typ"   { WriteTransTyp  $name }
-      "stm"   { WriteTransSTM  $name }
-      "lab" -
-      "esps"  { WriteTransOther $name }
-      "lola"  { WriteTransOther $name }
-      "phn" - "wrd" - "txt" -
-      "timit" { WriteTransOther $name }
-      "default" { WriteTransOther $name }
+   if {[info command ${format}::export] != ""} {
+      ${format}::export $name
+   } else {
+      error [format [Local "unsupported output format %s"] [namespace tail $format]]
    }
    DisplayMessage [format [Local "Transcription %s saved."] $name]
    return $format
@@ -701,7 +392,7 @@ proc WriteTrans {name {format ""}} {
 
 # Create new transcription. 
 # If audio file not given, ask through dialog box.
-proc NewTrans {{soundFile ""}} {
+proc NewTrans {{soundFile ""} {multiwav {}}} {
    global v
 
    if [catch {CloseTrans} err] return
@@ -709,7 +400,8 @@ proc NewTrans {{soundFile ""}} {
    if {[catch {
       if {$soundFile == "<empty>"} {
 	 EmptySignal
-      } elseif {$soundFile != "" && [file readable $soundFile]} {
+      } elseif {$soundFile != "" 
+		&& ([file readable $soundFile] || $v(sig,remote))} {
 	 Signal $soundFile
       } else {
 	 OpenAudioFile
@@ -719,6 +411,10 @@ proc NewTrans {{soundFile ""}} {
       EmptySignal
    }
 
+   # add multiple wav files from command line
+   eval MW_AddFile $multiwav
+
+   setChatMode 0
    set v(trans,name) ""
    UpdateShortName
    set v(trans,format) "trs"
@@ -736,29 +432,28 @@ proc SaveTrans {{as ""} {format ""}} {
    global v
 
    if {[GetSegmtNb seg0] <= 0} return
-   if {$format == "" || $format == "xml" || $format == "trs"} {
+   if {$format == ""} {
       set format "trs"
    }
    if {$v(trans,name) != "" && $as == "" && $v(trans,format) == $format} {
       set name $v(trans,name) 
    } else {
       if {$v(trans,name) == ""} {
-	 set base [$v(trans,root) getAttr "audio_filename"]
+	# default base name is first one from list given in episode tag
+	set base [lindex [GetFilename] 0]
       } else {
-	 set base [file root [file tail $v(trans,name)]]
-      }
-      if {$format == "trs"} {
-	 set types {{"Transcriptions"   {.trs .xml}}}
-      } else {
-	 set types [subst {{"Export format"   {.$format}}}]
+	set base [file root [file tail $v(trans,name)]]
       }
+      set exts [set ${format}::ext]
+      set ext [lindex $exts 0]
+      set types [list [list [set ${format}::msg] $exts]]
       lappend types {"All files"   {*}}
 
-      set name [tk_getSaveFile -filetypes $types -defaultextension ".$format" \
-		    -initialfile $base.$format -initialdir $v(trans,path) \
+      set name [tk_getSaveFile -filetypes $types -defaultextension $ext \
+		    -initialfile $base$ext -initialdir $v(trans,path) \
 		    -title  "Save transcription file $as"]
       if {$name != "" && [file extension $name] == ""} {
-	 append name ".$format"
+	 append name $ext
       }
    }
    if {$name != ""} {
@@ -772,8 +467,10 @@ proc SaveTrans {{as ""} {format ""}} {
       if [catch {
 	 WriteTrans $name $format
       } res] {
-	 tk_messageBox -message "$name not saved\n$res" -type ok -icon error
+	 tk_messageBox -message [format [Local "%s not saved !!"] $name] -type ok -icon error
 	 return "" 
+      } else {
+	  tk_messageBox -message [format [Local "%s saved !!"] $name] -type ok -icon info 
       }
       if {$format == "trs"} {
 	 set v(trans,name) $name
@@ -826,6 +523,7 @@ proc CloseTrans {{option save}} {
    ::speaker::init
    ::topic::init
    InitSegmt seg0 seg1 seg2 bg
+   DestroyLabels
    set v(trans,name) ""
    UpdateShortName
    InitModif
@@ -853,7 +551,8 @@ proc TransInfo {} {
 
 ################################################################
 
-# Normalize the transcription by "filling the holes" with sections or turns
+# Normalize the transcription by "filling the holes" with sections or turns,
+# check wrong boundary order
 # and creates empty data sections between non-contiguous breakpoints
 
 proc NormalizeTrans {} {
@@ -863,35 +562,46 @@ proc NormalizeTrans {} {
    set episode [$v(trans,root) getChilds "element" "Episode"]
 
    # Contrain sections to be a partition of the episode
-   set t1 $v(sig,min)
-   foreach sec [$episode getChilds "element" "Section"] {
-      set t2 [$sec getAttr "startTime"]
-      if {$t2 > $t1} {
+   #if {[namespace tail $v(trans,format)] != "hub4e"}
+   if {1} {
+     set t1 $v(sig,min)
+     foreach sec [$episode getChilds "element" "Section"] {
+       set t2 [$sec getAttr "startTime"]
+       if {$t2 < $t1} {
+	 $sec setAttr "startTime" $t1
+       } elseif {$t2 > $t1} {
 	 ::xml::element "Section" [list "type" "nontrans" "startTime" $t1 "endTime" $t2] -before $sec
-      }
-      set t1 [$sec getAttr "endTime"]
+       }
+       set t1 [$sec getAttr "endTime"]
+     }
+     # Don't add a new section up to the end, because we will synchronize
+     # the last breakpoint to the end of signal - else it would be done with:
+     #set t2 $v(sig,max)
+     #if {$t2 > $t1} {
+     #   ::xml::element "Section" [list "type" "nontrans" "startTime" $t1 "endTime" $t2] -in $episode
+     #}
    }
-   # Don't add a new section up to the end, because we will synchronize
-   # the last breakpoint to the end of signal - else it would be done with:
-   #set t2 $v(sig,max)
-   #if {$t2 > $t1} {
-   #   ::xml::element "Section" [list "type" "nontrans" "startTime" $t1 "endTime" $t2] -in $episode
-   #}
 
    foreach sec [$episode getChilds "element" "Section"] {
-      # Constrain turns to be a partition of each section
-      set t1 [$sec getAttr "startTime"]
-      foreach turn [$sec getChilds "element" "Turn"] {
-	 set t2 [$turn getAttr "startTime"]
-	 if {$t2 > $t1} {
+     # Constrain turns to be a partition of each section
+     if {1} {
+	set t1 [$sec getAttr "startTime"]
+	foreach turn [$sec getChilds "element" "Turn"] {
+	  set t2 [$turn getAttr "startTime"]
+	  if {$t2 < $t1} {
+	    $turn setAttr "startTime" $t1
+	  } elseif {$t2 > $t1} {
 	    ::xml::element "Turn" [list "startTime" $t1 "endTime" $t2] \
 		-before $turn
-	 }
-	 set t1 [$turn getAttr "endTime"]
-      }
-      set t2 [$sec getAttr "endTime"]
-      if {$t2 > $t1} {
-	 ::xml::element "Turn" [list "startTime" $t1 "endTime" $t2] -in $sec
+	  }
+	  set t1 [$turn getAttr "endTime"]
+	}
+	set t2 [$sec getAttr "endTime"]
+	if {$t2 < $t1} {
+	  $turn setAttr "endTime" $t2
+	} elseif {$t2 > $t1} {
+	  ::xml::element "Turn" [list "startTime" $t1 "endTime" $t2] -in $sec
+	}
       }
 
       foreach turn [$sec getChilds "element" "Turn"] {
@@ -901,6 +611,9 @@ proc NormalizeTrans {} {
 	 if {$sync == "" || [$sync getAttr "time"] > $t1} {
 	    ::xml::element "Sync" [list "time" $t1] -begin $turn
 	 }
+	 if {$sync != "" && [$sync getAttr "time"] < $t1} {
+	   $sync setAttr "time" $t1
+	 }
 	 # Create data between non-contiguous breakpoints
 	 foreach elem [$turn getChilds "element"] {
 	    set next [$elem getBrother "element"]
@@ -924,7 +637,8 @@ proc NormalizeTrans {} {
 	    set t1 $t2
 	 }
 	 # Convert data [...] to XML tags for .typ and old .xml format
-	 if {$v(trans,format) == "typ" || $v(convert_events)} {
+	 if {[namespace tail $v(trans,format)] == "typ"
+	     || $v(convert_events)} {
 	    foreach data [$turn getChilds "data"] {
 	       ConvertData $data
 	    }
@@ -994,10 +708,16 @@ proc DisplayTrans {} {
 
    set t0 [Synchro::NewTime $min]; set t6 $t0; set t5 $t0; set tprev $t0
    set bgTim $t0; set bgTxt ""; set bgId ""
+
+   if {$v(chatMode)} {
+     InsertEpisodeButton $episode
+   }
    foreach sec [$episode getChilds "element" "Section"] {
       set t1 [Synchro::NextTimeTag $sec "startTime"]
       # Editor button
-      InsertSectionButton $sec
+      if {!$v(chatMode)} {
+	InsertSectionButton $sec
+      }
 
       # register new section segment
       set t6 [Synchro::NextTimeTag $sec "endTime"]
@@ -1037,6 +757,10 @@ proc DisplayTrans {} {
 		  set id $chn
 		  set txt ""
 		  InsertSyncButton $id
+		  # update image for floating boundaries
+		  if {[Synchro::getElastic $t3]} {
+		    Synchro::updateSyncImage $t3
+		  }
 		  # for first Background breakpoint
 		  if {$bgId == ""} {
 		     set bgId $chn
@@ -1052,13 +776,23 @@ proc DisplayTrans {} {
 		  foreach {bgTxt img} [ReadBackAttrib $chn] {}
 		  InsertImage $chn $img
 	       }
+	       "Dependent" {
+		   InsertDependent $chn
+	       }
+	       "Header" {
+		   InsertHeader $chn
+	       }
+	       "Scope" {
+		   InsertScope $chn
+	       }
 	       "Event" - "Comment" {
 		  InsertEvent $chn
 		  append txt [StringOfEvent $chn]
 	       }
 	       "Who" {
 		  if {[$chn getAttr "nb"] > 1} {
-		     append txt "\n"
+		     # segment box is splitted vertically at form-feed
+		     append txt "\f"
 		  }
 		  InsertWho $chn
 	       }
@@ -1094,7 +828,7 @@ proc DisplayTrans {} {
    if {[info exists v(demo)]} {
       DestroyTextFrame
       DestroySegmentWidgets
-      CreateSegmentWidget .snd.w seg0 -fg $v(color,fg-sync) -full $v(color,bg-sync) -height 1 -high $v(color,hi-sync)
+      CreateSegmentWidget .snd.w seg0 "Demo" -fg $v(color,fg-sync) -full $v(color,bg-sync) -height 1 -high $v(color,hi-sync)
       destroy .demo
       frame .demo -bd 2 -relief raised
       pack .demo -expand true -fill both -side top
@@ -1110,6 +844,10 @@ proc DisplayTrans {} {
 
    # Create widgets if necessary
    CreateAllSegmentWidgets
+   # Only display requested ones
+   UpdateSegmtView
+   # Fancy colors if needed
+   ColorizeSpk
 
    HomeEditor
    DisplayMessage ""
@@ -1136,7 +874,8 @@ proc TextFromSync {bp} {
 	       "Who" {
 		  set nb [$tag getAttr "nb"]
 		  if {$nb > 1} {
-		     append txt "\n"
+		     # segment box is splitted vertically at form-feed
+		     append txt "\f"
 		  }
 	       }
 	       "Event" - "Comment" {
