@@ -1,4 +1,6 @@
-# Copyright (C) 1998, DGA - part of the Transcriber program
+# RCS: @(#) $Id: Waveform.tcl,v 1.5 2003/07/16 16:46:56 barras Exp $
+
+# Copyright (C) 1998-2000, DGA - part of the Transcriber program
 # distributed under the GNU General Public License (see COPYING file)
 
 ################################################################
@@ -56,12 +58,13 @@ proc CreateSoundFrame {f} {
    set v($wavfm,menu) [add_menu .menu$name [subst {
       {"Audio file"		cascade {
 	 {"Open audio file..." 	cmd {OpenAudioFile}}
+	 {"Add audio file..." 	cmd {OpenAudioFile add}}
+	 {""}
       }}
       {"Playback"		cascade {
 	 {"Play/Pause"		cmd {PlayOrPause}}
 	 {"Replay segment"	cmd {PlayCurrentSegmt}}
 	 {"Play around cursor"	cmd {PlayAround}}
-	 {"Backward and play"	cmd {PlayBefore}}
       }}
       {"Position"		cascade {
 	 {"Forward"	cmd {PlayForward +1}}
@@ -96,13 +99,14 @@ proc SwitchSoundFrame {f} {
    global v
 
    if {![winfo exists $f]} {
-      set wavfm [CreateSoundFrame $f]
-      ConfigWavfm $wavfm $v(sig,cmd)
-      CreateAllSegmentWidgets
+     set wavfm [CreateSoundFrame $f]
+     ConfigWavfm $wavfm
+     CreateAllSegmentWidgets
+     SetCursor [GetCursor]
    } elseif {[winfo ismapped $f]} {
-      pack forget $f
+     pack forget $f
    } else {
-      pack $f -fill x
+     pack $f -fill x
    }
 }
 
@@ -113,15 +117,15 @@ proc WavfmHeight {wavfm {val 1.0}} {
    $wavfm conf -height $v($wavfm,height)
 }
 
-proc ConfigWavfm {wavfm sound} {
+proc ConfigWavfm {wavfm} {
    global v
 
    set v($wavfm,left)    0
    set v($wavfm,size)    [setdef v($wavfm,resolution) 30]
-   if {!$v(shape,state)	&& $v($wavfm,size) > $v(shape,min)} {
+   if {$v(shape,cmd)=="" && $v($wavfm,size) > $v(shape,min)} {
       set v($wavfm,size) $v(shape,min)
    }
-   $wavfm config -sound $sound
+   $wavfm config -sound $v(sig,cmd) -shape $v(shape,cmd)
    SynchroWidgets $wavfm
 }
 
@@ -129,7 +133,7 @@ proc ConfigAllWavfm {} {
    global v
 
    foreach wavfm $v(wavfm,list) {
-      ConfigWavfm $wavfm $v(sig,cmd)
+      ConfigWavfm $wavfm
    }
 }
 
@@ -189,6 +193,9 @@ proc ScrollReso {wavfm cmd {val 0} {unit ""}} {
 
    if ($v(sig,len)<0) return;
 
+   if {[winfo exists .noshapemsg]} {
+          destroy .noshapemsg
+   }   
    # Try to keep cursor stable - else center of screen
    set curs $v(curs,pos)
    if {($curs >= $v($wavfm,left)) && ($curs <= $v($wavfm,right))} {
@@ -197,12 +204,11 @@ proc ScrollReso {wavfm cmd {val 0} {unit ""}} {
       set ratio 0.5
    }
    set t [expr $v($wavfm,left)+$v($wavfm,size)*$ratio]
-
    if {$cmd=="moveto"} {
-      if {$val>0} {set v($wavfm,size) [expr $val*$v(sig,len)]}
+     if {$val>0} {set v($wavfm,size) [expr $val*$v(sig,len)]}
    } elseif {$cmd=="scroll"} {
       if {$val>0} {
-	 set scale 1.1
+	set scale 1.1
       } else {
 	 set scale 0.9
       }
@@ -213,7 +219,8 @@ proc ScrollReso {wavfm cmd {val 0} {unit ""}} {
       set v($wavfm,size) 1e-5
    }
    # Min value for zoom if no shape is available
-   if {!$v(shape,state) && $v($wavfm,size) > $v(shape,min)} {
+   if {$v(sig,cmd) != "" && $v(shape,cmd) == "" && $v($wavfm,size) > $v(shape,min)} {
+      NoShapeMessage
       DisplayMessage "Lower resolution not allowed without signal shape"
       set v($wavfm,size) $v(shape,min)
    }
@@ -222,6 +229,21 @@ proc ScrollReso {wavfm cmd {val 0} {unit ""}} {
    SynchroWidgets $wavfm
 }
 
+proc NoShapeMessage {} {
+  set m .noshapemsg
+  if {[winfo exists $m]} {
+    destroy $m
+  }  
+  toplevel $m
+  wm title $m "Warning !"
+  wm geometry $m 250x100+500+350
+  label $m.l -text [Local "Resolution > 30\" impossible !\n\nCan't find the signal shape."]
+  button $m.b -text "Ok" -command {destroy .noshapemsg}
+  pack $m.l -fill both -expand true -padx 3m -pady 2m
+  pack $m.b -padx 3m -pady 2m
+  bell -displayof $m
+} 
+
 proc Resolution {reso {win ""}} {
    global v
 
@@ -248,7 +270,7 @@ proc ViewAll {{win ""}} {
    }
    set v($win,left) $v(sig,min)
    set v($win,size) $v(sig,len)
-   if {!$v(shape,state) && $v($win,size) > $v(shape,min)} {
+   if {$v(shape,cmd) == "" && $v($win,size) > $v(shape,min)} {
       set v($win,size) $v(shape,min)
    }
    SynchroWidgets $win
@@ -281,18 +303,21 @@ proc SetCursor {pos {hide 0}} {
    set v(curs,pos) $pos
    if {$hide == 1} {
       foreach wavfm $v(wavfm,list) {
-	 $wavfm config -cursor -1
+	 #$wavfm config -cursor -1
+	 $wavfm cursor -1
       }
       DisplayMessage "\t\t\t$v(sel,text)"
    } else {
       foreach wavfm $v(wavfm,list) {
-	 $wavfm config -cursor $pos
+	 #$wavfm config -cursor $pos
+	 $wavfm cursor $pos
 	 # View cursor
+	 set margin 0; #[expr $v($wavfm,size)*0.0]
 	 if {$pos > $v($wavfm,right)} {
-	    set v($wavfm,left) [expr $pos - $v($wavfm,size)]
+	    set v($wavfm,left) [expr $pos + $margin - $v($wavfm,size)]
 	    SynchroWidgets $wavfm
 	 } elseif {$pos < $v($wavfm,left) } {
-	    set v($wavfm,left) $pos
+	   set v($wavfm,left) [expr $pos - $margin]
 	    SynchroWidgets $wavfm
 	 }
       }
@@ -482,46 +507,48 @@ proc ExtendOldSelection {wavfm X} {
 }
 
 # View selection (or given interval)
-proc ViewSelection {{beg {}} {end {}}} {
+proc ViewSelection {{beg {}} {end {}} {mode "AUTO"} {ratio 0.1}} {
    global v
 
-   set win $v(tk,wavfm); # defaults to principal sound frame
-   if { ($beg != "" && $end != "") || [GetSelection beg end] } {
-      # If it can't fit completely on screen...
+  #set win $v(tk,wavfm); # defaults to principal sound frame
+  foreach win $v(wavfm,list) {
+    if { ($beg != "" && $end != "") || [GetSelection beg end] } {
+      set margin [expr $v($win,size)*$ratio]
       if {$end-$beg > $v($win,size)} {
-	 if {$end == $v(sig,max)} {
-            # center left side of last segment
-            set v($win,left) [expr $beg-$v($win,size)/2.0]
-	 } elseif {$end > $v($win,left) && $end < $v($win,right)} {
-            # show end
-            set v($win,left) [expr $end - $v($win,size)]
-         } elseif {$beg > $v($win,left) && $beg < $v($win,right) } {
-            # show begin
-            set v($win,left) $beg
-         } else {
-            # center
-            set v($win,left) [expr ($end+$beg-$v($win,size))/2.0]
-         }
+	# If it can't fit completely on screen...
+	if {$end == $v(sig,max)} {
+	  # center left side of last segment
+	  set v($win,left) [expr $beg-$v($win,size)/2.0]
+	} elseif {$mode == "END" || ($mode == "AUTO" && 
+		  $end > $v($win,left) && $end < $v($win,right))} {
+	  # show end
+	  set v($win,left) [expr $end + $margin - $v($win,size)]
+	} elseif {$mode == "BEGIN" || ($mode == "AUTO"
+                  && $beg > $v($win,left) && $beg < $v($win,right)) } {
+	  # show begin
+	  set v($win,left) [expr $beg - $margin]
+	} else {
+	  # center
+	  set v($win,left) [expr ($end+$beg-$v($win,size))/2.0]
+	}
+      } elseif {$end-$beg > (1-2*$ratio)*$v($win,size)} {
+	# center on the screen with a reduced margin
+	set v($win,left) [expr ($end+$beg-$v($win,size))/2.0]
       } else {
-	 if {$end-$beg < 0.8*$v($win,size)} {
-	    set margin [expr $v($win,size)*0.1]
-	 } else {
-	    set margin 0
-	 }
-	 if {$end > $v($win,right)} {
-	    # view 10% more if possible
-	    # show end
-	    set v($win,left) [expr $end + $margin - $v($win,size)]
-	 } elseif {$beg < $v($win,left) } {
-	    # show begin
-	    set v($win,left) [expr $beg - $margin]
-	 } else {
-	    # it's ok
-	    return
-	 }
+	if {$end > $v($win,right)} {
+	  # show end plus margin
+	  set v($win,left) [expr $end + $margin - $v($win,size)]
+	} elseif {$beg < $v($win,left) } {
+	  # show begin plus margin
+	  set v($win,left) [expr $beg - $margin]
+	} else {
+	  # it's ok
+	  continue
+	}
       }
       SynchroWidgets $win
-   }
+    }
+  }
 }
 
 ################################################################
@@ -557,3 +584,75 @@ proc UnZoom {{win ""}} {
       config_entry "Signal" "Unzoom selection" -state disabled
    }
 }
+# Save Audio Segment [as] 
+# Select the name, format and directory of the file to saved 
+# returns empty string if save failed (or was canceled)
+proc SaveAudioSegment {{as ""} {format ""}} {
+   global v 
+
+   set tmp "yes"
+   snack::sound player
+ 
+   if {$format == ""} {
+       set format ".wav"
+   }
+   if {$v(sel,begin) == $v(sel,end) } {
+      set tmp [tk_messageBox -icon warning -message [Local "No audio segment selected !"] \
+                                -title "Warning" -type ok]
+      return ""
+   } else {
+       player conf -file $v(sig,name)
+       if { $v(sig,open) == "0" } {
+           set v(sig,name) "empty" 
+           set tmp [tk_messageBox -icon question -message [Local "   No audio file opened !\nThis will create an empty\n audio file ! Really save ?"] \
+                                               -title "Warning" -type yesno]
+       }
+       if {$tmp == "yes"} {
+             set types {
+		    { "Wave file" {.wav}}
+		    { "AU file" {.au}}
+		    { "Sound file" {.snd}}
+		    { "SD file" {.sd}}
+		    { "SMP file" {.smp}}
+		    { "AIFF" {.aiff}}
+		    { "RAW file" {.raw}}
+		    { "All Files" {*}}
+	       }
+	       set base [file root [file tail $v(sig,name)]]
+	       set zone [concat [format "%6.2f" $v(sel,begin)]-[format "%-6.2f" $v(sel,end)]]
+	       set name [tk_getSaveFile -filetypes $types -defaultextension $format \
+					-initialfile "$base\_$zone$format" -initialdir $v(trans,path) \
+					-title "Save audio segment"]
+	       if {$name != "" && [file extension $name] == ""} {
+		   append name $format
+	       }
+	       if {$name == ""} {return}
+	} else {return}
+     }
+   if [catch {
+	 PauseAudio
+	 set wavename $v(sig,name)
+	 if {$wavename == ""} return
+
+	 # if possible, keep previously open sound file
+	 if {[player cget -file] != $wavename} {
+	     player conf -file $wavename -channels $v(sig,channels) -frequency $v(sig,rate) -skiphead $v(sig,header) -guessproperties 1
+	 }
+	 if { $v(sig,cmd) != "" } { 
+	     set rate [$v(sig,cmd) cget -frequency]
+	     player write $name -start [expr int($v(sel,begin)*$rate)] -end [expr int($v(sel,end)*$rate)]
+	 } else {
+		set time [expr int([format "%6.3f" [expr $v(sel,end) - $v(sel,begin)]]*16000)]
+		set f [snack::filter generator 0.0 0 0.0 sine $time] 
+		set s [snack::sound]
+		$s filter $f
+		$s write $name   
+	   }  
+   } res] {
+	 tk_messageBox -message [format [Local "%s not saved !!"] $name] -type ok -icon error
+	 return "" 
+      } else {
+              tk_messageBox -message [format [Local "%s saved !!"] $name] -type ok -icon info
+        }
+   return $name
+}
